@@ -1,4 +1,3 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/common/xf86Helper.c,v 1.136 2004/01/27 01:31:45 dawes Exp $ */
 
 /*
  * Copyright (c) 1997-2003 by The XFree86 Project, Inc.
@@ -70,7 +69,6 @@
 static int xf86ScrnInfoPrivateCount = 0;
 
 
-#ifdef XFree86LOADER
 /* Add a pointer to a new DriverRec to xf86DriverList */
 
 _X_EXPORT void
@@ -143,6 +141,32 @@ xf86DeleteInputDriver(int drvIndex)
     xf86InputDriverList[drvIndex] = NULL;
 }
 
+InputDriverPtr
+xf86LookupInputDriver(const char *name)
+{
+    int i;
+
+    for (i = 0; i < xf86NumInputDrivers; i++) {
+       if (xf86InputDriverList[i] && xf86InputDriverList[i]->driverName &&
+           xf86NameCmp(name, xf86InputDriverList[i]->driverName) == 0)
+           return xf86InputDriverList[i];
+    }
+    return NULL;
+}
+
+InputInfoPtr
+xf86LookupInput(const char *name)
+{
+    InputInfoPtr p;
+
+    for (p = xf86InputDevs; p != NULL; p = p->next) {
+        if (strcmp(name, p->name) == 0)
+            return p;
+    }
+
+    return NULL;
+}
+
 _X_EXPORT void
 xf86AddModuleInfo(ModuleInfoPtr info, pointer module)
 {
@@ -172,7 +196,6 @@ xf86DeleteModuleInfo(int idx)
 	xf86ModuleInfoList[idx] = NULL;
     }
 }
-#endif
 
 
 /* Allocate a new ScrnInfoRec in xf86Screens */
@@ -199,11 +222,7 @@ xf86AllocateScreen(DriverPtr drv, int flags)
 
     xf86Screens[i]->drv = drv;
     drv->refCount++;
-#ifdef XFree86LOADER
     xf86Screens[i]->module = DuplicateModule(drv->module, NULL);
-#else
-    xf86Screens[i]->module = NULL;
-#endif
     /*
      * set the initial access state. This will be modified after PreInit.
      * XXX Or should we do it some other place?
@@ -260,10 +279,8 @@ xf86DeleteScreen(int scrnIndex, int flags)
 
     xf86OptionListFree(pScrn->options);
 
-#ifdef XFree86LOADER
     if (pScrn->module)
 	UnloadModule(pScrn->module);
-#endif
 
     if (pScrn->drv)
 	pScrn->drv->refCount--;
@@ -321,11 +338,7 @@ xf86AllocateInput(InputDriverPtr drv, int flags)
 
     new->drv = drv;
     drv->refCount++;
-#ifdef XFree86LOADER
     new->module = DuplicateModule(drv->module, NULL);
-#else
-    new->module = NULL;
-#endif
     new->next = xf86InputDevs;
     xf86InputDevs = new;
     return new;
@@ -352,10 +365,8 @@ xf86DeleteInput(InputInfoPtr pInp, int flags)
 	pInp->free(pInp, 0);
 #endif
 
-#ifdef XFree86LOADER
     if (pInp->module)
 	UnloadModule(pInp->module);
-#endif
 
     if (pInp->drv)
 	pInp->drv->refCount--;
@@ -2381,17 +2392,12 @@ xf86GetVersion()
 _X_EXPORT CARD32
 xf86GetModuleVersion(pointer module)
 {
-#ifdef XFree86LOADER
     return (CARD32)LoaderGetModuleVersion(module);
-#else
-    return 0;
-#endif
 }
 
 _X_EXPORT pointer
 xf86LoadDrvSubModule(DriverPtr drv, const char *name)
 {
-#ifdef XFree86LOADER
     pointer ret;
     int errmaj = 0, errmin = 0;
 
@@ -2400,15 +2406,11 @@ xf86LoadDrvSubModule(DriverPtr drv, const char *name)
     if (!ret)
 	LoaderErrorMsg(NULL, name, errmaj, errmin);
     return ret;
-#else
-    return (pointer)1;
-#endif
 }
 
 _X_EXPORT pointer
 xf86LoadSubModule(ScrnInfoPtr pScrn, const char *name)
 {
-#ifdef XFree86LOADER
     pointer ret;
     int errmaj = 0, errmin = 0;
 
@@ -2417,9 +2419,6 @@ xf86LoadSubModule(ScrnInfoPtr pScrn, const char *name)
     if (!ret)
 	LoaderErrorMsg(pScrn->name, name, errmaj, errmin);
     return ret;
-#else
-    return (pointer)1;
-#endif
 }
 
 /*
@@ -2428,9 +2427,7 @@ xf86LoadSubModule(ScrnInfoPtr pScrn, const char *name)
 _X_EXPORT pointer
 xf86LoadOneModule(char *name, pointer opt)
 {
-#ifdef XFree86LOADER
     int errmaj, errmin;
-#endif
     char *Name;
     pointer mod;
     
@@ -2452,13 +2449,9 @@ xf86LoadOneModule(char *name, pointer opt)
 	return NULL;
     }
 
-#ifdef XFree86LOADER
     mod = LoadModule(Name, NULL, NULL, NULL, opt, NULL, &errmaj, &errmin);
     if (!mod)
 	LoaderErrorMsg(NULL, Name, errmaj, errmin);
-#else
-    mod = (pointer)1;
-#endif
     xfree(Name);
     return mod;
 }
@@ -2470,7 +2463,7 @@ xf86UnloadSubModule(pointer mod)
      * This is disabled for now.  The loader isn't smart enough yet to undo
      * relocations.
      */
-#if defined(XFree86LOADER) && 0
+#if 0
     UnloadSubModule(mod);
 #endif
 }
@@ -2478,59 +2471,28 @@ xf86UnloadSubModule(pointer mod)
 _X_EXPORT Bool
 xf86LoaderCheckSymbol(const char *name)
 {
-#ifdef XFree86LOADER
     return LoaderSymbol(name) != NULL;
-#else
-    return TRUE;
-#endif
 }
 
+/* These two are just ABI stubs, they don't do anything in dlloader world */
 _X_EXPORT void
 xf86LoaderReqSymLists(const char **list0, ...)
 {
-#ifdef XFree86LOADER
-    va_list ap;
-
-    va_start(ap, list0);
-    LoaderVReqSymLists(list0, ap);
-    va_end(ap);
-#endif
 }
 
 _X_EXPORT void
 xf86LoaderReqSymbols(const char *sym0, ...)
 {
-#ifdef XFree86LOADER
-    va_list ap;
-
-    va_start(ap, sym0);
-    LoaderVReqSymbols(sym0, ap);
-    va_end(ap);
-#endif
 }
 
 _X_EXPORT void
 xf86LoaderRefSymLists(const char **list0, ...)
 {
-#ifdef XFree86LOADER
-    va_list ap;
-
-    va_start(ap, list0);
-    LoaderVRefSymLists(list0, ap);
-    va_end(ap);
-#endif
 }
 
 _X_EXPORT void
 xf86LoaderRefSymbols(const char *sym0, ...)
 {
-#ifdef XFree86LOADER
-    va_list ap;
-
-    va_start(ap, sym0);
-    LoaderVRefSymbols(sym0, ap);
-    va_end(ap);
-#endif
 }
 
 
@@ -2614,7 +2576,7 @@ xf86SetSilkenMouse (ScreenPtr pScreen)
     /*
      * XXX quick hack to report correctly for OSs that can't do SilkenMouse
      * yet.  Should handle this differently so that alternate async methods
-     * like Xqueue work correctly with this too.
+     * work correctly with this too.
      */
     pScrn->silkenMouse = useSM && xf86SIGIOSupported();
     if (serverGeneration == 1)
@@ -2677,6 +2639,7 @@ xf86ConfigIsaEntity(ScrnInfoPtr pScrn, int scrnFlag, int entityIndex,
     if (!pEnt->active) {
 	xf86ConfigIsaEntityInactive(pEnt, i_chip, res, init,  enter,
 				    leave,  private);
+	xfree(pEnt);
 	return pScrn;
     }
 
@@ -2714,6 +2677,7 @@ xf86ConfigPciEntity(ScrnInfoPtr pScrn, int scrnFlag, int entityIndex,
     if (!pEnt->active) {
 	xf86ConfigPciEntityInactive(pEnt, p_chip, res, init,  enter,
 				    leave,  private);
+	xfree(pEnt);
 	return pScrn;
     }
 
@@ -2755,6 +2719,7 @@ xf86ConfigFbEntity(ScrnInfoPtr pScrn, int scrnFlag, int entityIndex,
 
     if (!pEnt->active) {
 	xf86ConfigFbEntityInactive(pEnt, init,  enter, leave,  private);
+	xfree(pEnt);
 	return pScrn;
     }
 
