@@ -1,3 +1,3 @@
 #define MFBPOLYGLYPHBLT xf1bppPolyGlyphBltInvert
 #define OPEQ ^=
-#include "../../../mfb/mfbplygblt.c"
+#include "../../../../mfb/mfbplygblt.c"
